// Copyright 2010 Google Inc. All Rights Reserved.

/**
 * @fileoverview Implementation of the communication protocol between the webapp
 * and the native app.
 *
 * @author dominicl@google.com (Dominic Leung)
 */
(function() {
  if (!navigator || navigator.gmailNativeMessaging) {
    return;
  }

  var messagingApi = {
    apiVersion: 1
  };

  /**
   * Updates the unread count on the badge.
   * @param {number} unreadCount The unread count for the update.
   * @param {boolean=} opt_isFreshData Whether the update originated from
   *     a completed network refresh.
   */
  messagingApi.updateUnreadCount = function(unreadCount, opt_isFreshData) {
    navigator.googleInternal.dispatch('gmailnativemessaging',
        'updateUnreadCount', {
           inboxCount: unreadCount,
           isFreshData: !!opt_isFreshData
        });
  };

  /**
   * Show the native contacts picker.
   */
  messagingApi.showNativeContactsPicker = function(requestCallback) {
    this.currentContactsRequestCallback_ = requestCallback;
    navigator.googleInternal.dispatch('gmailnativemessaging',
        'showContactsPicker', {});
  };

  /**
   * Called by native code when the contacts selection request succeeded.
   * @param {string} email The e-mail address that was selected.
   * @param {string} name The name of the contact that was selected.
   */
  messagingApi.contactsCallback_ = function(email, name) {
    if (this.currentContactsRequestCallback_) {
      this.currentContactsRequestCallback_(email, name);
    }
  };

  messagingApi.updateLabelsProto = function(proto) {
    navigator.googleInternal.dispatch('gmailnativemessaging',
        'updateLabels', { labelsData: proto });
  };

  messagingApi.setTLQuery = function(query, labelId, isSearch,
      isPriorityInbox) {
    navigator.googleInternal.dispatch('gmailnativemessaging',
        'setTLQuery', { query: query, labelId: labelId, isSearch: isSearch,
        isPriorityInbox: isPriorityInbox });
  };

  /**
   * Called by the javascript code to send the encoded avatar photo to the
   * native app.
   * @param {string} dataUrl The encoded image data as a dataUrl.
   */
  messagingApi.setNativeMenuAvatar = function(dataUrl) {
    navigator.googleInternal.dispatch('gmailnativemessaging',
        'setMenuAvatar', { dataString: dataUrl });
  };

  /**
   * Called by the javascript code to open the side drawer. For iPhone and
   * legacy iPad, this is the labels view. For IPAD_PLUS this is the side nav
   * bar. The function name is maintained for backwards compatibility.
   */
  messagingApi.toggleLabelsView = function() {
    // TODO(carm): Consider renaming toggleLabelsView to toggleSideDrawerView.
    navigator.googleInternal.dispatch('gmailnativemessaging',
        'toggleSideDrawerView');
  };

  /**
   * Called from web to initiate user feedback.
   * @param {string} logs The javascript logs to attach to the feedback.
   * @param {boolean} dogfood True if the feedback is from a dogfooder.
   * @param {!Object.<string, string>|undefined} productDataJson The serialized
   *     product-specific data to send with the feedback, or undefined if there
   *     isn't any.
   */
  messagingApi.startFeedback = function(logs, dogfood, productDataJson) {
    navigator.googleInternal.dispatch('gmailnativemessaging', 'startFeedback',
        {logs: logs, dogfood: !!dogfood, productDataJson: productDataJson});
  };

  // The JS callback for returning user agent data.
  messagingApi.userAgentDataRequestCallback_;

  /**
   * Called from the web to gather user agent data for user feedback purposes.
   * @param {!function(!Object)} requestCallback The JS function to call with
   *     user agent data supplied.
   */
  messagingApi.fetchUserAgentData = function(requestCallback) {
    messagingApi.userAgentDataRequestCallback_ = requestCallback;
    navigator.googleInternal.dispatch('gmailnativemessaging',
       'fetchUserAgentData');
  };

  messagingApi.toggleLabelsSwiping = function() {
    navigator.googleInternal.dispatch('gmailnativemessaging',
        'toggleLabelsSwiping');
  };

  messagingApi.webappUpdateReady = function() {
    navigator.googleInternal.dispatch('gmailnativemessaging',
        'webappUpdateReady');
  };

  messagingApi.settingsViewDone = function() {
    navigator.googleInternal.dispatch('gmailnativemessaging',
        'settingsViewDone');
  };

  messagingApi.settingsViewHidden = function() {
    navigator.googleInternal.dispatch('gmailnativemessaging',
        'settingsViewHidden');
  };

  messagingApi.settingsViewShown = function() {
    navigator.googleInternal.dispatch('gmailnativemessaging',
        'settingsViewShown');
  };

  messagingApi.showAppIntegrationSettings = function() {
    navigator.googleInternal.dispatch('gmailnativemessaging',
        'showAppIntegrationSettings');
  };
  // The JS callback for returning queried contacts.
  messagingApi.currentAddressBookRequestCallback_;

  messagingApi.searchNativeAddressBook = function(requestCallback,
      queryStringData, requestId, maxResults) {
    // TODO(donnadupuis): Store callback function in a map keyed by requestId.
    messagingApi.currentAddressBookRequestCallback_ = requestCallback;
    navigator.googleInternal.dispatch('gmailnativemessaging',
        'searchNativeAddressBookForQuery', {queryString: queryStringData,
        requestId: requestId, maxResults: maxResults});
  };

  /**
   * Called by native code when address book request succeeded.
   * @param {string} addressBook The address book as a string.
   * @param {string} queryString The contacts search query.
   * @param {string} requestId The ID of the native contacts request.
   */
  messagingApi.addressBookCallback_ = function(addressBook, queryString,
      requestId) {
    if (messagingApi.currentAddressBookRequestCallback_) {
      messagingApi.currentAddressBookRequestCallback_(addressBook, queryString,
          requestId);
      messagingApi.currentAddressBookRequestCallback_ = undefined;
    }
  };

  /**
   * Called by superpudu when the view changes.
   * @param {string} oldViewType The type of the view about to be hidden
   *     (should be of type savannah.common.ViewType).
   * @param {string} newViewType The type of the view about to be shown
   *     (should be of type savannah.common.ViewType)
   */
  messagingApi.showView = function(oldViewType, newViewType) {
    messagingApi.showViewWithCallback(oldViewType, newViewType,
        undefined /* webViewOffScreenCallback */);
  }

  /**
   * Called by superpudu when the view changes.
   * @param {string} oldViewType The type of the view about to be hidden
   *     (should be of type savannah.common.ViewType).
   * @param {string} newViewType The type of the view about to be shown
   *     (should be of type savannah.common.ViewType)
   * @param {function()|undefined} webViewOffScreenCallback A callback to call
   *     once the webview is positioned off screen.
   */
  messagingApi.showViewWithCallback =
      function(oldViewType, newViewType, webViewOffScreenCallback) {
    messagingApi.webViewOffScreenCallback_ = webViewOffScreenCallback;
    navigator.googleInternal.dispatch('gmailnativemessaging', 'showView',
    { oldViewType: oldViewType, newViewType: newViewType });
  };

  /**
   * Calls the callback stored in showViewWithCallback, if it exists.
   */
  messagingApi.invokeWebViewOffScreenCallback = function() {
    if (messagingApi.webViewOffScreenCallback_) {
      messagingApi.webViewOffScreenCallback_();
    }
  };

  /**
   * Called by JavaScript after a view has been refreshed by invoking
   * the native pull down to refresh functionality.
   * @param {boolean} didRefresh true if the refresh actually happened, false if
   *     the refresh did not happen for some reason (for example, if the app was
   *     offline).
   */
  messagingApi.hidePullDownToRefresh = function(didRefresh) {
    navigator.googleInternal.dispatch('gmailnativemessaging',
        'hidePullDownToRefresh', { didRefresh: !!didRefresh });
  }

  /** Called by the Javascript to log a database size impression. */
  messagingApi.logDatabaseSizeImpression =
      function(databaseName, impressionId) {
    navigator.googleInternal.dispatch('gmailnativemessaging',
        'logDatabaseSizeImpression', { databaseName: databaseName,
        impressionId: impressionId});
  };

  /**
   * Called by the Javascript to log a reachability impression plus xhr error
   * code.
   * @param {string} errCode The error code accompanying the failed xhr.
   */
  messagingApi.logNativeNetworkStatusImpression = function(errCode) {
    navigator.googleInternal.dispatch('gmailnativemessaging',
        'logNativeNetworkStatusImpression', { errCode: errCode });
  };

 /**
  * Called by Javascript to log a roundtrip time impression with reachability.
  * @param {number} time The round trip time in milliseconds.
  */
 messagingApi.logRoundtripNetworkStatusImpression = function(time) {
   navigator.googleInternal.dispatch('gmailnativemessaging',
       'logRoundtripNetworkStatusImpression', { time: time });
 };

  /**
   * Called by the JS to show/hide the top toolbar when the user is searching.
   * @param {boolean} focussed True if the search is focused, false when
   *     blurred.
   */
  messagingApi.setSearchFocused = function(focused) {
    navigator.googleInternal.dispatch('gmailnativemessaging',
        'setSearchFocused', { focused: focused });
  };

  /**
   * Called by the Javascript to determine if it will get called when native
   * transitions complete. This function does nothing and exists for the JS to
   * know if it should expect a call when transitions complete.
   * @return {boolean} This method always returns true because versions of
   *     native with this function will notify on transition complete.
   */
  messagingApi.doesNotifyOnTransitionComplete = function() {
    return true;
  };

  /**
   * Called by JS when an avatar upload response has been received, indicating
   * failure or success of the upload.
   * @param {boolean} succeeded Whether the request succeeded.
   */
  messagingApi.avatarUploadCompleted = function(succeeded) {
    navigator.googleInternal.dispatch('gmailnativemessaging',
        'avatarUploadCompleted', { succeeded: succeeded });
  };

  /**
   * Called by JS when the compose view is opened.
   */
  messagingApi.composeViewOpened = function() {
    navigator.googleInternal.dispatch('gmailnativemessaging',
        'composeViewOpened');
  };

  /**
   * Called by JS whent the compose view is hidden but the context remains
   * composing a message (such as creating a doodle as an attachment).
   */
  messagingApi.composeViewSuspended = function() {
    navigator.googleInternal.dispatch('gmailnativemessaging',
        'composeViewSuspended');
  };

  /**
   * Called by JS when the compose view has been closed.
   * @param {boolean} emailSending Whether the email from the compose view is
   *     being sent. If not, the email has been discarded or saved as draft.
   * @param {number=} opt_requestId The id of the email send request.
   */
  messagingApi.composeViewClosed = function(emailSending, opt_requestId) {
    navigator.googleInternal.dispatch('gmailnativemessaging',
        'composeViewClosed',
        { emailSending: emailSending, requestId: opt_requestId });
  };

  /**
   * Called by JS when an email has been sent.
   * @param {boolean} succeeded Whether the email was sent successfuly.
   * @param {number} requestId The id of the email send request.
   */
  messagingApi.emailSent = function(succeeded, requestId) {
    navigator.googleInternal.dispatch('gmailnativemessaging',
        'emailSent', { succeeded: succeeded, requestId: requestId });
  };

  /**
   * Called by JS to indicate that it has done something to invalidate a
   * screenshot used in a fast transition.
   */
  messagingApi.invalidateFastTransitionScreenshot = function (viewType) {
    navigator.googleInternal.dispatch('gmailnativemessaging',
        'invalidateFastTransitionScreenshot', { viewType: viewType });
  }

  /**
   * Called by JS when an a request to the notifications server should be made
   * to get the configurable notification setting. The response will be sent
   * back through the callback asynchronously.
   * @param {!function(string, string)} requestCallback The JS function callback
   *     to use when the request has completed, successful or not. The first
   *     parameter is a serialized NotificationSetting proto for the user's
   *     current setting, if the request succeeded. If the request fails, the
   *     second parameter will contain the error message.
   */
  messagingApi.getConfigurableNotification = function(requestCallback) {
    messagingApi.getConfigurableNotificationRequestCallback_ = requestCallback;
    navigator.googleInternal.dispatch('gmailnativemessaging',
                                      'getConfigurableNotification');
  };

  /**
   * Calls the callback for the setConfigurableNotifications request, if one
   * exists.
   * @param {!string} type Serialized string representing the setting type.
   * @param {!string} error Error message if the request failed, empty string if
   *     it succeeded.
   */
  messagingApi.invokeGetConfigurableNotificationCallback =
      function(type, error) {
    if (messagingApi.getConfigurableNotificationRequestCallback_) {
      messagingApi.getConfigurableNotificationRequestCallback_(type, error);
    }
  };

  /**
   * Called by JS when an a request to the notifications server should be made
   * to set the configurable notification setting. The response will be sent
   * back through the callback asynchronously.
   * @param {!string} notificationSetting String form of the NotificationType to
   *     send back.
   * @param {!function(string)} requestCallback The JS function callback to use
   *     when the request has completed, successful or not. The parameter is a
   *     containing the error message if the request fails, and undefined if it
   *     succeeds.
   */
  messagingApi.setConfigurableNotification =
      function(notificationSetting, requestCallback) {
    messagingApi.setConfigurableNotificationRequestCallback_ = requestCallback;
    navigator.googleInternal.dispatch('gmailnativemessaging',
        'setConfigurableNotification',
        { notificationSetting: notificationSetting });
  };

  /**
   * Calls the callback for the setConfigurableNotifications request, if one
   * exists.
   * @param {!string} error Error message if the request failed, empty string if
   *     it succeeded.
   */
  messagingApi.invokeSetConfigurableNotificationCallback = function(error) {
    if (messagingApi.setConfigurableNotificationRequestCallback_) {
      messagingApi.setConfigurableNotificationRequestCallback_(error);
    }
  };

  /**
   * Adds the url regex to the list of whitelisted urls that are not redirected
   * to the external webview.
   * @param {!string} urlRegex The regex for evaluating the url.
   * @param {!function()} requestCallback The JS function callback to use when
   *     the request has completed.
   */
  messagingApi.addWhitelistedUrl =
      function(urlRegex, requestCallback) {
    messagingApi.addWhitelistedUrlRequestCallback_ = requestCallback;
    navigator.googleInternal.dispatch('gmailnativemessaging',
        'addWhitelistedUrl', { urlRegex : urlRegex });
  };

  /**
   * Calls the callback for the addWhitelistedUrl request, if one exists.
   */
  messagingApi.invokeAddWhitelistedUrlCallback = function() {
    if (messagingApi.addWhitelistedUrlRequestCallback_) {
      messagingApi.addWhitelistedUrlRequestCallback_();
      messagingApi.addWhitelistedUrlRequestCallback_ = undefined;
    }
  };

  /**
   * Native handler when an attachment is clicked in the JS.
   * @param {string} attachments Serialized proto array representing all
   *     attachments attached to the current conversation message.
   * @param {number} index The index of the selected attachment.
   */
  messagingApi.onAttachmentClick = function(attachments, index) {
    navigator.googleInternal.dispatch('gmailnativemessaging',
        'onAttachmentClick', { attachments: attachments, index: index });
  };

  messagingApi.openDriveSelector = function() {
    navigator.googleInternal.dispatch('gmailnativemessaging',
        'openDriveSelector');
  };

  // The JS callback for returning queried contacts.
  messagingApi.fixDriveAttachmentPermissionsRequestCallback_;

  /**
   * Checks and fixes any missing permissions for the attached drive files.
   * @param {!string} driveList Serialized array list of drive ID's.
   * @param {!string} emailList Serialized array list of recepient emails.
   * @param {!function()} requestCallback The JS function callback to use when
   *     the request has completed.
   */
  messagingApi.fixDriveAttachmentPermissions =
      function(driveList, emailList, requestCallback) {
    messagingApi.fixDriveAttachmentPermissionsRequestCallback_ =
        requestCallback;
    navigator.googleInternal.dispatch('gmailnativemessaging',
        'fixDriveAttachmentPermissions',
        { driveList: driveList, emailList: emailList });
  };

  /**
   * Calls the callback for the fixDriveAttachmentPermissions request, if one
   * exists.
   */
  messagingApi.invokeFixDriveAttachmentPermissionsCallback = function() {
    if (messagingApi.fixDriveAttachmentPermissionsRequestCallback_) {
      messagingApi.fixDriveAttachmentPermissionsRequestCallback_();
      fixDriveAttachmentPermissionsRequestCallback_ = undefined;
    }
  };

  navigator.gmailNativeMessaging = messagingApi;
})();
