// Copyright 2012 Google Inc. All Rights Reserved.

/**
 * @fileoverview Implementation of the communication protocol between the top
 * toolbar and the native app.
 * @author danbrotherston@google.com (Daniel Brotherston)
 */

(function() {
  if (!navigator || navigator.topToolbarNativeMessaging) {
    return;
  }

  var messagingApi = {
    apiVersion: 1
  };

 /**
  * Called by the JS to indicate that the alternate buttons should be enabled
  * or disabled on the specified view's toolbar.
  * @param {string} viewType The view to change the toolbar for.
  * @param {boolean} enabled Whether to enable or disable the alternate buttons.
  */
  messagingApi.setAlternateButtonsEnabled = function(viewType, enabled) {
    navigator.googleInternal.dispatch('toptoolbarmessaging',
        'setAlternateButtonsEnabled',
        { viewType: viewType, enabled: !!enabled });
  };

 /**
  * Called by the JS to set the toolbar view title for a view.
  * @param {string} viewType View to set the toolbar title for.
  * @param {string} toolbarTitle Title to set on the specified view's toolbar.
  */
  messagingApi.setViewToolbarTitle = function(viewType, toolbarTitle) {
    navigator.googleInternal.dispatch('toptoolbarmessaging',
        'setViewToolbarTitle',
        { viewType: viewType, toolbarTitle: toolbarTitle });
  };

 /**
  * Called by the JS to set the TL view toolbar title.
  * @param {string} viewType View to set the toolbar title for.
  * @param {string} queryTitle Query title.
  * @param {number} viewCount Count of messages in the view.
  */
  messagingApi.setTlViewToolbarTitle = function(
      viewType, queryTitle, viewCount) {
    navigator.googleInternal.dispatch('toptoolbarmessaging',
        'setTlViewToolbarTitle',
        { viewType: viewType, queryTitle: queryTitle, viewCount: viewCount });
  };

 /**
  * Called by the JS to set the back button text on a view.
  * @param {string} viewType View to set the toolbar title for.
  * @param {string} backButtonTitle Title to set on the specified view's back
  *     button.
  */
  messagingApi.setBackButtonTitle = function(viewType, backButtonTitle) {
    navigator.googleInternal.dispatch('toptoolbarmessaging',
        'setBackButtonTitle',
        { viewType: viewType, backButtonTitle: backButtonTitle });
  };

  /**
   * Called by the JS to enable/disable the button with the given control type.
   * @param {string} controlType The control type that the button maps to.
   * @param {boolean} enabled True to enable, false to disable.
   */
  messagingApi.setButtonEnabledWithControlType =
      function(controlType, enabled) {
    navigator.googleInternal.dispatch('toptoolbarmessaging',
        'setButtonEnabledWithControlType',
        { controlType: controlType, enabled: !!enabled });
  };

  /**
   * Called by the Javascript to update the actions that can be taken on the
   * current conversation.
   * @param {string} config Serialized proto array representing the
   *     savannah.bamenu.BatchActionsMenu.Config to render.
   */
  messagingApi.convActionsRender = function(config) {
    navigator.googleInternal.dispatch('toptoolbarmessaging',
        'convActionsRender', { config: config });
  };

  /**
   * Called by the JS to show/hide the conversation toolbar's more menu.
   * @param {boolean} show True to show, false to hide.
   */
  messagingApi.cvShowMoreMenu = function(show) {
    navigator.googleInternal.dispatch('toptoolbarmessaging',
        'cvShowMoreMenu', { show: !!show });
  };

  /**
   * Called by the JS to ask a view to consider itself "busy", displaying
   * a spinner and disabling controls, as appropriate.
   * @param {string} viewType View to mark as busy.
   * @param {boolean} busy True if busy, false if not busy.
   */
  messagingApi.setViewBusy = function(viewType, busy) {
    navigator.googleInternal.dispatch('toptoolbarmessaging',
        'setViewBusy', { viewType: viewType, busy: !!busy });
  };

  /**
   * Called by the JS to show/hide the top toolbar.
   * @param {boolean} hidden True to hide, False to show.
   */
  messagingApi.setNavigationBarHidden = function(hidden) {
    navigator.googleInternal.dispatch('toptoolbarmessaging',
        'setNavigationBarHidden', { hidden: hidden });
  };

  /**
   * Called by the JS to enable/disable the top toolbar.
   * @param {boolean} enabled True to enable the toolbar, false to disable the
   *     toolbar.
   */
  messagingApi.setTopToolbarEnabled = function(enabled) {
    navigator.googleInternal.dispatch('toptoolbarmessaging',
        'setTopToolbarEnabled', { enabled: enabled });
  };

  /**
   * Called by the JS to enable/disable all toolbar buttons.
   * @param {boolean} enabled True to enable the buttons, false to disable the
   *     buttons.
   */
  messagingApi.setButtonsEnabledForView = function(viewType, enabled) {
    navigator.googleInternal.dispatch('toptoolbarmessaging',
        'setButtonsEnabledForView', { viewType: viewType, enabled: enabled });
  };

  navigator.topToolbarNativeMessaging = messagingApi;
})();
