// Copyright 2012 Google Inc. All Rights Reserved.

/**
 * @fileoverview Javascript to support warm welcome.
 * As much content as possible that is not localized is created here.
 * There must be no localizable content in this file.
 * @author sforsyth@google.com
 * @author kennethkwan@google.com
 */

/** @constructor */
function WarmWelcome() {}

/**
 * @type {!WarmWelcome}
 */
var warmWelcome = new WarmWelcome();

/**
 * Enum for device types.
 * @enum {number}
 */
warmWelcome.DeviceType = {
  IPHONE: 0,
  IPAD: 1,
  IPHONE_5: 2
};

/**
 * Horizontal space occupied by a progress dot in pixels.
 * @const
 * @private
 * @type {number}
 */
warmWelcome.PROGRESS_DOT_WIDTH_ = 14;

/**
 * The window width in pixels over which to use the iPad images.
 * @const
 * @private
 * @type {number}
 */
warmWelcome.IPAD_SCREEN_WIDTH_CUTOFF_ = 500;

/**
 * The window height in pixels over which to use the iPhone5 images instead of
 * the iphone images.
 * @const
 * @private
 * @type {number}
 */
warmWelcome.IPHONE_5_SCREEN_HEIGHT_CUTOFF_ = 500;

/**
 * The height of the background image for the iPhone in pixels.
 * @const
 * @private
 * @type {number}
 */
warmWelcome.IPHONE_BACKGROUND_IMAGE_HEIGHT_ = 256;

/**
 * The height of the background image for the iPhone5 in pixels.
 * @const
 * @private
 * @type {number}
 */
warmWelcome.IPHONE_5_BACKGROUND_IMAGE_HEIGHT_ = 296;

/**
 * The height of the background image for the iPad in pixels.
 * @const
 * @private
 * @type {number}
 */
warmWelcome.IPAD_BACKGROUND_IMAGE_HEIGHT_ = 398;

/**
 * The path to the file used as a background image.
 * @const
 * @private
 * @type {string}
 */
warmWelcome.BACKGROUND_IMAGE_PATH_ = '../bg_mountains.png';

/**
 * The CSS class to apply to an element for transitions on webkit transforms.
 * @const
 * @private
 * @type {string}
 */
warmWelcome.CSS_TRANSITION_CLASS_ = 'CSS_TRANSITION';

/**
 * The CSS class to apply to the page that is the current page.
 * @const
 * @private
 * @type {string}
 */
warmWelcome.CSS_CURRENT_PAGE_CLASS_ = 'CSS_CURRENT_PAGE';

/**
 * The minimum speed in pixels/s for a swipe to be recognized as a page
 * transition for finger tracking.
 * @const
 * @private
 * @type {number}
 */
warmWelcome.FINGER_TRACKING_MIN_SWIPE_SPEED_ = 100;

/**
 * The minimum transition duration in seconds for a transition due to a finger
 * tracking swipe.
 * @const
 * @private
 * @type {number}
 */
warmWelcome.FINGER_TRACKING_MIN_TRANSITION_DURATION_ = 0.5;

/**
 * The element containing the background image. Used for parallax scrolling.
 * @private
 * @type {Element}
 */
warmWelcome.backgroundImage_;

/**
 * The URL used to report the page number back to the native client.
 * The actual number is appended to the URL to form the fragment.
 * @const
 * @private
 * @type {string}
 */
warmWelcome.PAGE_REPORT_URL_ = 'google://welcomePage#';

/**
 * The URL used to report to the native client that the HTML has finished
 * loading.
 * @const
 * @private
 * @type {string}
 */
warmWelcome.FINISHED_LOADING_URL_ = 'google://finishedLoading';

/**
 * True if the current language is RTL, false otherwise.
 * @private
 * @type {boolean|undefined}
 */
warmWelcome.isRTL_;

/**
 * The HTML fragment that precedes a video in the content.
 * @private
 * @type {string}
 */
warmWelcome.VIDEO_PREFIX_ =
    '<video controls="controls" webkit-playsinline class="CSS_SIMPLE_FRAME">' +
    '<source src="';

/**
 * The HTML fragment that follows a video in the content.
 * @private
 * @type {string}
 */
warmWelcome.VIDEO_POSTFIX_ = '" type="video/x-m4v"></video>';

/**
 * The HTML fragment that precedes an image in the content.
 * @private
 * @type {string}
 */
warmWelcome.IMAGE_PREFIX_ =
    '<div class="CSS_DISPLAY_IMAGE_FRAME"><div class="CSS_DISPLAY_IMAGE">';

/**
 * The HTML fragment that follows an image in the content.
 * @private
 * @type {string}
 */
warmWelcome.IMAGE_POSTFIX_ = '</div></div>';

/**
 * The HTML fragment that precedes the text content.
 * @private
 * @type {string}
 */
warmWelcome.TEXT_PREFIX_ = '<div class="CSS_DISPLAY_TEXT">';

/**
 * The HTML fragment that precedes the text content for RTL languages.
 * @private
 * @type {string}
 */
warmWelcome.TEXT_PREFIX_RTL_ = '<div class="CSS_DISPLAY_TEXT" dir="rtl">';

/**
 * The HTML fragment that follows the text content.
 * @private
 * @type {string}
 */
warmWelcome.TEXT_POSTFIX_ = '</div>';

/** @typedef {{image: (string|undefined), video: (string|undefined)}} */
warmWelcome.Media;

/**
 * @typedef {{page: string,
 *     iPhone: warmWelcome.Media,
 *     iPad: warmWelcome.Media,
 *     iPhone5: warmWelcome.Media}}
 */
warmWelcome.PageDescription;

/**
 * The pages used in the welcome.
 * @private
 * @type {!Array.<!warmWelcome.PageDescription>}
 */
warmWelcome.pages_ = [];

/**
 * The pages available for the welcome.
 * @const
 * @private
 * @type {!Array.<!warmWelcome.PageDescription>}
 */
warmWelcome.AVAILABLE_PAGES_ = [
  { 'page': 'start.html',
    'iPhone': {'image': '<img src="../homescreen.png" width="250">'},
    'iPad': {'image': '<img src="../homescreen_iPad.png" width="450">'},
    'iPhone5': {
      'image':
          '<img src="../homescreen_iPhone5.png" width="250">'
    }
  },
  { 'page': 'multilogin.html',
    'iPhone': {
      'image':
          '<img src="../multilogin.png" width="250" class="CSS_PAN_IPHONE">'
    },
    'iPad': {'image': '<img src="../multilogin_iPad.png" width="450">'},
    'iPhone5': {
      'image':
          '<img src="../multilogin.png" width="250" class="CSS_PAN_IPHONE">'
    }
  },
  { 'page': 'search.html',
    'iPhone': {
        'simpleFrame': true,
        'image':
            '<img src="../search.png" width="250" class="CSS_SIMPLE_FRAME">'},
    'iPad': {'image': '<img src="../search_iPad.png" width="450">'},
    'iPhone5': {
      'image':
          '<img src="../search_iPhone5.png" width="250">'
    }
  },
  { 'page': 'calendar.html',
    'iPhone': {'image': '<img src="../calendar.png" width="250">'},
    'iPad': {'image': '<img src="../calendar_iPad.png" width="450">'},
    'iPhone5': {
      'image':
          '<img src="../calendar_iPhone5.png" width="250" ' +
          'class="CSS_PAN_IPHONE">'
    }
  },
  { 'page': 'compose.html',
    'iPhone': {'video': '../scribble_iPhone.mp4'},
    'iPad': {'video': '../scribble_iPad.mov'},
    'iPhone5': {'video': '../scribble_iPhone.mp4'},
  },
  { 'page': 'swipe.html',
    'iPhone': {
      'simpleFrame': true,
      'image': '<img src="../swipe.png" width="250" class="CSS_SIMPLE_FRAME">'
    },
    'iPad': { /* no iPad version */ },
    'iPhone5': {
      'image': '<img src="../swipe_iphone5.png" width="250">'
    }
  },
  { 'page': 'sectionStart.html',
    'iPhone': {'image': '<img src="../sectionedStart_iPhone.png" width="250">'},
    'iPad': {'image': '<img src="../sectionedStart_iPad.png" width="450">'},
    'iPhone5': {
      'image': '<img src="../sectionedStart_iPhone5.png" width="250">'
    }
  },
  { 'page': 'sectionLabels.html',
    'iPhone': {
      'image': '<img src="../sectionedLabels_iPhone.png" width="250">'
    },
    'iPad': {'image': '<img src="../sectionedLabels_iPad.png" width="450">'},
    'iPhone5': {
      'image': '<img src="../sectionedLabels_iPhone5.png" width="250">'
    }
  },
  { 'page': 'sectionNotifications.html',
    'iPhone': {'image': '<img src="../sectionedNotif_iPhone.png" width="250">'},
    'iPad': {'image': '<img src="../sectionedNotif_iPad.png" width="450">'},
    'iPhone5': {
      'image': '<img src="../sectionedNotif_iPhone5.png" width="250">'
    }
  },
  { 'page': 'sso.html',
    'iPhone': {'image': '<img src="../sso_iphone.png" width="250">'},
    'iPad': {'image': '<img src="../sso_ipad.png" width="450">'},
    'iPhone5': {'image': '<img src="../sso_iphone5.png" width="250">'}
  },
  { 'page': 'notifications.html',
    'iPhone': {'image': '<img src="../notifications_iphone.png" width="250">'},
    'iPad': {'image': '<img src="../notifications_ipad.png" width="450">'},
    'iPhone5': {'image': '<img src="../notifications_iphone5.png" width="250">'}
  },
  { 'page': 'apps.html',
    'iPhone': {'image': '<img src="../apps_iphone.png" width="250">'},
    'iPad': {'image': '<img src="../apps_ipad.png" width="450">'},
    'iPhone5': {'image': '<img src="../apps_iphone5.png" width="250">'}
  }];

/**
 * The current device type.
 * @private
 * @type {!warmWelcome.DeviceType|undefined}
 */
warmWelcome.currDevice_;

/**
 * The content of the pages used in the welcome.
 * @private
 * @type {!Array.<string>}
 */
warmWelcome.pageContent_ = [];

/**
 * The divs for the pages used in the welcome.
 * @private
 * @type {!Array.<Element>}
 */
warmWelcome.pageDivs_ = [];

/**
 * The button used to move left a page.
 * @private
 * @type {!Element}
 */
warmWelcome.leftButton_ = document.createElement('a');

/**
 * The button used to move right a page.
 * @private
 * @type {!Element}
 */
warmWelcome.rightButton_ = document.createElement('a');

/**
 * The button used to advance to the second page of the warm welcome.
 * @private
 * @type {!Element}
 */
warmWelcome.startButton_ = document.createElement('a');

/**
 * The button used to close the warm welcome.
 * @private
 * @type {!Element}
 */
warmWelcome.closeButton_ = document.createElement('a');

/**
 * The index of the currently displayed page. Undefined until page loads for the
 * first time.
 * @private
 * @type {number|undefined}
 */
warmWelcome.curPageIndex_;

/**
 * True iff navigating between pages is allowed.
 * @private
 * @type {boolean}
 */
warmWelcome.navigationDisabled_ = false;

/**
 * A hidden element used to preload images.
 * @private
 * @type {!Element}
 */
warmWelcome.imageBuffer_ = document.createElement('div');

/**
 * The progress dot representing the active page of content.
 * @private
 * @type {!Element|undefined}
 */
warmWelcome.activePageIndicator_;

/**
 * True iff finger tracking should be ignored.
 * @type {boolean|undefined}
 * @private
 */
warmWelcome.ignoreFingerTracking_;

/**
 * Hide a button by prepending the CSS_HIDDEN style if not already present.
 * @param {!Element} button The button to hide.
 * @private
 */
warmWelcome.hideButton_ = function(button) {
  if (button.className.indexOf('CSS_HIDDEN') == -1) {
    button.className = 'CSS_HIDDEN ' + button.className;
  }
};

/**
 * Set the styles of the navigation buttons on each page.
 * Most pages show just left and right arrows, but the first and last pages
 * require special handling.
 * The first page must have a button to close the tour early and the arrow to
 * advance to the next page is replaced with a "Start Tour" button.
 * The last page replaces the next arrow with a close button.
 * Because RTL languages reverse the direction of the pages and arrows, the
 * special buttons on the first and last pages must be drawn on different sides
 * depending on language direction.
 * @param {number} pageNumber The page number of the page being shown.
 * @private
 */
warmWelcome.setButtonStylesForPage_ = function(pageNumber) {
  if (pageNumber == 1) {
    warmWelcome.rightButton_.className = 'CSS_BTN_RIGHT CSS_HIDDEN';
    warmWelcome.leftButton_.className = 'CSS_BTN_LEFT CSS_HIDDEN';
    if (warmWelcome.isRTL_) {
      warmWelcome.closeButton_.className =
        'CSS_BUTTON CSS_RIGHT CSS_BTN_STANDARD CSS_BTN_ENABLED';
      warmWelcome.startButton_.className =
        'CSS_BUTTON CSS_LEFT CSS_BTN_PRIMARY CSS_BTN_ENABLED';
    } else {
      warmWelcome.closeButton_.className =
        'CSS_BUTTON CSS_LEFT CSS_BTN_STANDARD CSS_BTN_ENABLED';
      warmWelcome.startButton_.className =
        'CSS_BUTTON CSS_RIGHT CSS_BTN_PRIMARY CSS_BTN_ENABLED';
    }
  } else if (pageNumber == warmWelcome.pages_.length) {
    warmWelcome.hideButton_(warmWelcome.startButton_);
    if (warmWelcome.isRTL_) {
      warmWelcome.closeButton_.className =
        'CSS_BUTTON CSS_LEFT CSS_BTN_PRIMARY CSS_BTN_ENABLED';
      warmWelcome.leftButton_.className = 'CSS_BTN_LEFT CSS_HIDDEN';
      warmWelcome.rightButton_.className = 'CSS_BTN_RIGHT CSS_BTN_ENABLED';
    } else {
      warmWelcome.closeButton_.className =
        'CSS_BUTTON CSS_RIGHT CSS_BTN_PRIMARY CSS_BTN_ENABLED';
      warmWelcome.leftButton_.className = 'CSS_BTN_LEFT CSS_BTN_ENABLED';
      warmWelcome.rightButton_.className = 'CSS_BTN_RIGHT CSS_HIDDEN';
    }
  } else {
    warmWelcome.hideButton_(warmWelcome.startButton_);
    warmWelcome.hideButton_(warmWelcome.closeButton_);
    warmWelcome.leftButton_.className = 'CSS_BTN_LEFT CSS_BTN_ENABLED';
    warmWelcome.rightButton_.className = 'CSS_BTN_RIGHT CSS_BTN_ENABLED';
  }
};

/**
 * Move the background image, potentially with an animation.
 * @param {number} toPage The index of the page to animate to.
 * @param {boolean} animate True iff the background move should be animated.
 * @param {number} opt_transitionDuration Optional transition duration in
 *     seconds.
 * @private
 */
warmWelcome.moveBackground_ =
    function(toPage, animate, opt_transitionDuration) {
  warmWelcome.enableTransitions_(warmWelcome.backgroundImage_, animate);
  warmWelcome.restoreDefaultTransitionDuration_(warmWelcome.backgroundImage_);
  if (opt_transitionDuration) {
    warmWelcome.setTransitionDuration_(warmWelcome.backgroundImage_,
        opt_transitionDuration);
  }
  var shiftAmountPerPage =
      (warmWelcome.backgroundImage_.width - window.innerWidth) /
      (warmWelcome.pages_.length - 1);
  var newLeft = -1 * (shiftAmountPerPage * toPage);
  var webkitTransform = warmWelcome.createStringForXTranslation_(newLeft);
  warmWelcome.backgroundImage_.style.webkitTransform = webkitTransform;
};

/**
 * Create and set the animation used to animate the content divs.
 * @param {number} fromPage The index of the page to animate from.
 * @param {number} toPage The index of the page to animate to.
 * @param {number|undefined} opt_transitionDuration Optional transition duration
 *     in seconds.
 * @private
 */
warmWelcome.animateContent_ =
    function(fromPage, toPage, opt_transitionDuration) {
  var width = window.innerWidth;
  var currentPage = warmWelcome.pageDivs_[fromPage];
  var nextPage = warmWelcome.pageDivs_[toPage];

  // Clear all previous transition duration properties.
  warmWelcome.restoreDefaultTransitionPropertiesOnAllPages_();

  warmWelcome.enableTransitions_(currentPage, true);
  warmWelcome.enableTransitions_(nextPage, true);

  // Overwrite the default transition duration if it was supplied.
  if (opt_transitionDuration) {
    warmWelcome.setTransitionDuration_(currentPage, opt_transitionDuration);
    warmWelcome.setTransitionDuration_(nextPage, opt_transitionDuration);
  }

  // Going left, transitioning to the right.
  if (fromPage > toPage) {
    nextPage.style.webkitTransform =
        warmWelcome.createStringForXTranslation_(0);
    currentPage.style.webkitTransform =
        warmWelcome.createStringForXTranslation_(width);

  // Going right, transitioning to the left.
  } else if (fromPage < toPage) {
    currentPage.style.webkitTransform =
        warmWelcome.createStringForXTranslation_(-width);
    nextPage.style.webkitTransform =
        warmWelcome.createStringForXTranslation_(0);

  // No page change, just reset positions.
  } else {
    currentPage.style.webkitTransform =
        warmWelcome.createStringForXTranslation_(0);
    if (fromPage > 0) {
      var leftPage = warmWelcome.pageDivs_[fromPage - 1];
      warmWelcome.enableTransitions_(leftPage, true);
      if (opt_transitionDuration) {
        warmWelcome.setTransitionDuration_(leftPage, opt_transitionDuration);
      }
      leftPage.style.webkitTransform =
          warmWelcome.createStringForXTranslation_(-width);
    }
    if (fromPage < warmWelcome.pageDivs_.length - 1) {
      var rightPage = warmWelcome.pageDivs_[fromPage + 1];
      warmWelcome.enableTransitions_(rightPage, true);
      if (opt_transitionDuration) {
        warmWelcome.setTransitionDuration_(rightPage, opt_transitionDuration);
      }
      rightPage.style.webkitTransform =
          warmWelcome.createStringForXTranslation_(width);
    }
  }
};

/**
 * Create and set the animation used to animate the page indicators.
 * @param {number} toPage The index of the page to animate to.
 * @private
 */
warmWelcome.animatePageIndicator_ = function(toPage) {
  var activeClasses = 'CSS_PROGRESS CSS_CURRENT';
  // If an activePageIndicator_ already exists, we should fade it out before
  // fading in the new one.
  if (warmWelcome.activePageIndicator_) {
    activeClasses += ' CSS_FADE_IN';
    warmWelcome.activePageIndicator_.className = 'CSS_PROGRESS CSS_FADE_OUT';
  }
  warmWelcome.activePageIndicator_ = document.getElementById('p' + toPage);
  warmWelcome.activePageIndicator_.className = activeClasses;
};

/**
 * Navigate to the correct page in the warm welcome.
 * @param {number} pageIndex The index of the page to change to.
 * @param {number|undefined} opt_transitionDuration Optional transition duration
 *     in seconds.
 */
warmWelcome.setPage = function(pageIndex, opt_transitionDuration) {
  if (warmWelcome.navigationDisabled_) {
    return;
  }

  // Do not allow the user to move the views while transitions are in progress.
  warmWelcome.ignoreFingerTracking_ = true;

  if (warmWelcome.curPageIndex_ == undefined) {
    // This is the first load. Instead of animating the content, we just set the
    // positions for the necessary divs.
    var width = window.innerWidth;
    for (var i = 0; i < warmWelcome.pageDivs_.length; i++) {
      var div = warmWelcome.pageDivs_[i];
      if (i < pageIndex) {
        div.style.webkitTransform =
            warmWelcome.createStringForXTranslation_(-width);
      } else if (i == pageIndex) {
        div.style.webkitTransform =
            warmWelcome.createStringForXTranslation_(0);
      } else {
        div.style.webkitTransform =
            warmWelcome.createStringForXTranslation_(width);
      }
    }
  } else {
    warmWelcome.animateContent_(
        warmWelcome.curPageIndex_, pageIndex, opt_transitionDuration);
  }

  // If the width is not set, it means the image has not fully loaded. In that
  // case, perform the background move after the image is loaded, but without an
  // animation.
  if (!warmWelcome.backgroundImage_.width) {
    warmWelcome.backgroundImage_.onload = function() {
      warmWelcome.moveBackground_(pageIndex, false /* animate */);
    }
  } else {
    warmWelcome.moveBackground_(pageIndex, true /* animate */,
        opt_transitionDuration);
  }

  warmWelcome.resetCurrentPageClass_(pageIndex);

  var pageNumber = warmWelcome.getLogicalPageNumber_(pageIndex);
  warmWelcome.setButtonStylesForPage_(pageNumber);

  // Only need to perform these next steps if the page changes.
  if (warmWelcome.curPageIndex_ != pageIndex) {
    warmWelcome.animatePageIndicator_(pageIndex);

    warmWelcome.reportPageNumber_(pageNumber);
    warmWelcome.curPageIndex_ = pageIndex;
  }

  warmWelcome.ignoreFingerTracking_ = false;
};

/**
 * Converts the array index to a 1-based page number that represents the
 * order of pages as seen by the user. Array indices are 0-based,
 * and RTL languages also reverse the items in the array.
 * @param {number} pageIndex The page index to convert.
 * @private
 */
warmWelcome.getLogicalPageNumber_ = function(pageIndex) {
  var pageNumber = pageIndex + 1;
  if (warmWelcome.isRTL_) {
    pageNumber = warmWelcome.pages_.length - pageIndex;
  }
  return pageNumber;
};

/**
 * Report that the HTML is finished loading to the native app.
 * @private
 */
warmWelcome.finishLoading_ = function() {
  // Wrap the call in a setTimeout so that any pending DOM manipulations in the
  // current execution stack are completed first.
  window.setTimeout(function() {
      window.location = warmWelcome.FINISHED_LOADING_URL_;
  }, 0);
};

/**
 * Report the current page number to the native app.
 * @param {number} pageNumber The 1-based page number to report.
 * @private
 */
warmWelcome.reportPageNumber_ = function(pageNumber) {
  window.location = warmWelcome.PAGE_REPORT_URL_ + pageNumber;
};

/**
 * Set the caption and target of a button.
 * Called from Objective-C.
 * @param {string} id The DOM id of the button to change.
 * @param {string} caption The text to display to the user.
 * @param {string|undefined} opt_targetUrl URL to load when the button is
 *    clicked. An undefined or empty URL disables the button.
 */
warmWelcome.setButtonValues = function(id, caption, opt_targetUrl) {
  var targetButton = document.getElementById(id);
  var newText = document.createTextNode(caption);
  targetButton.replaceChild(newText, targetButton.firstChild);
  targetButton.href = opt_targetUrl ? opt_targetUrl : '#';
};

/**
 * Button handler to advance one page to the right.
 * @private
 */
warmWelcome.goRight_ = function() {
  warmWelcome.setPage(Math.min(warmWelcome.curPageIndex_ + 1,
                               warmWelcome.pages_.length - 1));
  return false;
};

/**
 * Button handler to go back one page to the left.
 * @private
 */
warmWelcome.goLeft_ = function() {
  warmWelcome.setPage(Math.max(warmWelcome.curPageIndex_ - 1, 0));
  return false;
};

/**
 * Disable the button.
 * @param {!Element} button .
 * @private
 */
warmWelcome.disableButton_ = function(button) {
  button.className =
      button.className.replace('CSS_BTN_ENABLED', 'CSS_BTN_DISABLED');
  button.onclick = null;
};

/**
 * Button handler to disable navigation.
 * @private
 */
warmWelcome.disableNavigation_ = function() {
  warmWelcome.navigationDisabled_ = true;
  warmWelcome.disableButton_(warmWelcome.closeButton_);
  warmWelcome.disableButton_(warmWelcome.leftButton_);
  warmWelcome.disableButton_(warmWelcome.rightButton_);
  warmWelcome.disableButton_(warmWelcome.startButton_);

  // Also disable finger tracking so that the user can no longer swipe through
  // pages.
  warmWelcome.ignoreFingerTracking_ = true;

  return true;
};

/**
 * Build the navigation pane used to show and manipulate place in the welcome.
 * The navigation pane also contains a link to close the welcome.
 * @return {!Element} The element representing the root of the DOM tree
 *     representing the navigation pane.
 * @private
 */
warmWelcome.buildNavigationPane_ = function() {
  var navigationPane = document.createElement('div');
  navigationPane.className = 'CSS_NAVIGATION_CONTAINER';

  var progressPane = document.createElement('div');
  var progressPaneSize =
      warmWelcome.PROGRESS_DOT_WIDTH_ * warmWelcome.pages_.length;
  progressPane.style.width = progressPaneSize + 'px';
  progressPane.className = 'CSS_PROGRESS_CONTAINER';
  navigationPane.appendChild(progressPane);

  for (var i = 0; i < warmWelcome.pages_.length; i++) {
    var progressMarker = document.createElement('div');
    progressMarker.setAttribute('id', 'p' + i);
    progressMarker.className = 'CSS_PROGRESS';
    progressPane.appendChild(progressMarker);
  }

  warmWelcome.closeButton_.id = 'inbox';
  warmWelcome.closeButton_.href = '#';
  var closeText = document.createTextNode('');
  warmWelcome.closeButton_.appendChild(closeText);
  warmWelcome.closeButton_.onclick = warmWelcome.disableNavigation_;
  navigationPane.appendChild(warmWelcome.closeButton_);

  warmWelcome.rightButton_.id = 'rightButton';
  warmWelcome.rightButton_.onclick = warmWelcome.goRight_;
  navigationPane.appendChild(warmWelcome.rightButton_);

  warmWelcome.leftButton_.id = 'leftButton';
  warmWelcome.leftButton_.onclick = warmWelcome.goLeft_;
  navigationPane.appendChild(warmWelcome.leftButton_);

  warmWelcome.startButton_.id = 'startTour';
  var startText = document.createTextNode('');
  warmWelcome.startButton_.appendChild(startText);
  navigationPane.appendChild(warmWelcome.startButton_);
  warmWelcome.startButton_.onclick = warmWelcome.isRTL_ ?
      warmWelcome.goLeft_ : warmWelcome.goRight_;

  return navigationPane;
};

/**
 * Construct the document body using translated content.
 * @private
 */
warmWelcome.buildDocument_ = function() {
  var body = document.body;

  // Setting up the background image.
  var backgroundImageContainer = document.createElement('div');
  backgroundImageContainer.className = 'CSS_BACKGROUND_CONTAINER';
  warmWelcome.backgroundImage_ = document.createElement('img');
  warmWelcome.backgroundImage_.src = warmWelcome.BACKGROUND_IMAGE_PATH_;
  switch (warmWelcome.currDevice_) {
    case warmWelcome.DeviceType.IPHONE:
      warmWelcome.backgroundImage_.height =
          warmWelcome.IPHONE_BACKGROUND_IMAGE_HEIGHT_;
      break;
    case warmWelcome.DeviceType.IPAD:
      warmWelcome.backgroundImage_.height =
          warmWelcome.IPAD_BACKGROUND_IMAGE_HEIGHT_;
      break;
    case warmWelcome.DeviceType.IPHONE_5:
      warmWelcome.backgroundImage_.height =
          warmWelcome.IPHONE_5_BACKGROUND_IMAGE_HEIGHT_;
      break;
    default:
      // This would be very bad. Use a zero height if we ever end up here since
      // we would have no idea what the right height should be.
      warmWelcome.backgroundImage_.height = 0;
      break;
  }
  backgroundImageContainer.appendChild(warmWelcome.backgroundImage_);
  body.appendChild(backgroundImageContainer);

  // Image buffer.
  warmWelcome.imageBuffer_.className = 'CSS_CONTENT_HIDDEN';
  body.appendChild(warmWelcome.imageBuffer_);

  // Setting up the content divs.
  var width = window.innerWidth;
  for (var i = 0; i < warmWelcome.pageContent_.length; i++) {
    var div = document.createElement('div');
    div.innerHTML = warmWelcome.pageContent_[i];
    div.className = 'CSS_CONTENT_DIV';
    warmWelcome.pageDivs_[i] = div;
    body.appendChild(div);
  }

  // Bottom navigation section.
  body.appendChild(warmWelcome.buildNavigationPane_());

  // Enable active style for button press
  document.addEventListener('touchstart', function() {}, false);
};

/**
 * Load an image prior to its being displayed to improve appearance by
 * appending it to a buffer that is never displayed.
 * @param {string} image A HTML img tag that represents the image to load.
 * @private
 */
warmWelcome.preloadImage_ = function(image) {
  var container = document.createElement('div');
  container.innerHTML = image;
  warmWelcome.imageBuffer_.appendChild(container);
};

/**
 * Load a video prior to its being displayed to improve appearance by
 * appending it to a buffer that is never displayed.
 * @param {string} video The relative path to the video file.
 * @private
 */
warmWelcome.preloadVideo_ = function(video) {
  var container = document.createElement('video');
  var source = document.createElement('source');
  source.source = video;
  container.appendChild(source);
  warmWelcome.imageBuffer_.appendChild(container);
};

/**
 * Generate a text string representing the HTML for the content of the page.
 * @param {!warmWelcome.PageDescription} pageInfo A collection of resources for
 *     generating the page.
 * @param {!warmWelcome.DeviceType} device The kind of device the page is being
 *     displayed on.
 * @return {string}
 * @private
 */
warmWelcome.generatePageText_ = function(pageInfo, device) {
  var platformInfo;
  switch (device) {
    case warmWelcome.DeviceType.IPHONE:
      platformInfo = pageInfo.iPhone;
      break;
    case warmWelcome.DeviceType.IPAD:
      platformInfo = pageInfo.iPad;
      break;
    case warmWelcome.DeviceType.IPHONE_5:
      platformInfo = pageInfo.iPhone5;
      break;
    default:
      // Report no error - it is possible that there is no visual content
      // and this avoids adding an empty frame.
      break;
  }

  var page = '';
  if (platformInfo.video) {
    page = page + warmWelcome.VIDEO_PREFIX_ +
        platformInfo.video + warmWelcome.VIDEO_POSTFIX_;
    warmWelcome.preloadVideo_(platformInfo.video);
  } else if (platformInfo.image) {
    if (platformInfo.simpleFrame) {
      page = page + platformInfo.image;
    } else {
      page = page + warmWelcome.IMAGE_PREFIX_ +
          platformInfo.image + warmWelcome.IMAGE_POSTFIX_;
    }
    warmWelcome.preloadImage_(platformInfo.image);
  }

  var pageRequest = new XMLHttpRequest();
  pageRequest.open('GET', pageInfo.page, false /* asynchronous */);
  pageRequest.send();
  if (warmWelcome.isRTL_) {
    page = page + warmWelcome.TEXT_PREFIX_RTL_;
  } else {
    page = page + warmWelcome.TEXT_PREFIX_;
  }
  page = page + pageRequest.responseText + warmWelcome.TEXT_POSTFIX_;
  return page;
};

/**
 * Handles movement of the content as a user moves their finger on the screen.
 * @param {number} translation The net horizontal translation of the finger in
 *     pixels.
 * @param {number|undefined} opt_endVelocity The horizontal velocity when the
 *     finger is lifted from the screen, undefined while the finger remains on
 *     the screen.
 */
warmWelcome.handleFingerTracking = function(translation, opt_endVelocity) {
  if (warmWelcome.ignoreFingerTracking_) {
    return;
  }

  var backgroundImage = warmWelcome.backgroundImage_;
  var currentPage = warmWelcome.pageDivs_[warmWelcome.curPageIndex_];
  if (warmWelcome.curPageIndex_ > 0) {
    var leftPage = warmWelcome.pageDivs_[warmWelcome.curPageIndex_ - 1];
  }
  if (warmWelcome.curPageIndex_ < warmWelcome.pageDivs_.length - 1) {
    var rightPage = warmWelcome.pageDivs_[warmWelcome.curPageIndex_ + 1];
  }

  var pageWidth = window.innerWidth;
  var numPages = warmWelcome.pages_.length;
  var shiftAmountPerPage = (backgroundImage.width - pageWidth) / (numPages - 1);

  // Clear all transition durations in case they are not needed.
  warmWelcome.restoreDefaultTransitionDuration_(backgroundImage);
  warmWelcome.restoreDefaultTransitionPropertiesOnAllPages_();

  // User released finger with some volecity. Need to determine if we transition
  // to a new page or if we reset the current page.
  if (opt_endVelocity != undefined) {

    // Fast swipe from left to right and there is a page to the left.
    if (opt_endVelocity > warmWelcome.FINGER_TRACKING_MIN_SWIPE_SPEED_ &&
        warmWelcome.curPageIndex_ > 0) {
      var transitionDuration =
          warmWelcome.timeToReachNextPage_(opt_endVelocity, translation);
      transitionDuration = Math.min(
          warmWelcome.FINGER_TRACKING_MIN_TRANSITION_DURATION_,
          transitionDuration);
      warmWelcome.setPage(warmWelcome.curPageIndex_ - 1, transitionDuration);

    // Fast swipe from right to left and there is a page to the right.
    } else if (opt_endVelocity < -warmWelcome.FINGER_TRACKING_MIN_SWIPE_SPEED_
        && warmWelcome.curPageIndex_ < numPages - 1) {
      var transitionDuration =
          warmWelcome.timeToReachNextPage_(opt_endVelocity, translation);
      transitionDuration = Math.min(
          warmWelcome.FINGER_TRACKING_MIN_TRANSITION_DURATION_,
          transitionDuration);
      warmWelcome.setPage(warmWelcome.curPageIndex_ + 1, transitionDuration);

    // Swipe from left to right covering 1/3 of page and there is a page to the
    // left.
    } else if (translation > (pageWidth / 3) && warmWelcome.curPageIndex_ > 0) {
      warmWelcome.setPage(warmWelcome.curPageIndex_ - 1);

    // Swipe from right to left covering 1/3 of page and there is a page to the
    // right.
    } else if (translation < -1 * (pageWidth / 3) &&
        warmWelcome.curPageIndex_ < numPages - 1) {
      warmWelcome.setPage(warmWelcome.curPageIndex_ + 1);

      // Swipe was slow and short, reset the current page.
    } else {
      warmWelcome.setPage(warmWelcome.curPageIndex_);
    }

  // User's finger is still on the screen, so move the content to follow their
  // finger.
  } else {
    // Remove the TRANSITON CSS class so that the transition is instant.
    warmWelcome.enableTransitions_(backgroundImage, false);

    // When pulling the first page to the right or last page to the left, do not
    // allow the background to scroll.
    var backgroundShiftAmount;
    if (!(translation < 0 && warmWelcome.curPageIndex_ == numPages - 1 ||
        translation > 0 && warmWelcome.curPageIndex_ == 0)) {
      backgroundShiftAmount = shiftAmountPerPage *
          ((translation / pageWidth) - warmWelcome.curPageIndex_);
      backgroundImage.style.webkitTransform =
          warmWelcome.createStringForXTranslation_(backgroundShiftAmount);
    }

    currentPage.style.webkitTransform =
        warmWelcome.createStringForXTranslation_(translation);
    if (leftPage) {
      leftPage.style.webkitTransform =
          warmWelcome.createStringForXTranslation_(translation - pageWidth);
    }
    if (rightPage) {
      rightPage.style.webkitTransform =
          warmWelcome.createStringForXTranslation_(translation + pageWidth);
    }
  }
};

/**
 * Calculates the time required to reach the side of the page if the page
 * continues to travel at the given velocity having already moved the given
 * translation. To be used as a transition duration.
 * @param {number} velocity Speed in pixels/sec. Positive is in the right
 *     direction.
 * @param {number} translation Distance that has already been travelled in
 *     pixels.
 * @return {number} Time in seconds to reach the end of the page.
 * @private
 */
warmWelcome.timeToReachNextPage_ = function(velocity, translation) {
  var width = window.innerWidth;
  if (velocity > 0) {
    return ((width - translation) / velocity);
  } else if (velocity < 0) {
    return -1 * ((width + translation) / velocity);
  }
};

/**
 * Creates and returns a string to be used for a webkit transform horizontal
 * translation.
 * @param {number} x The number of pixels for the horizontal translation.
 * @return {string} The CSS string for the translate.
 * @private
 */
warmWelcome.createStringForXTranslation_ = function(x) {
  return 'translate3d(' + x + 'px,0,0)';
};

/**
 * Sets a custom transition duration for a DOM element.
 * @param {!Element} element The element to set the transition duration on.
 * @param {number} duration The transition duration in seconds.
 * @private
 */
warmWelcome.setTransitionDuration_ = function(element, duration) {
  element.style['-webkit-transition-duration'] = duration + 's';
};

/**
 * Removes any custom transition duration on a DOM element to restore the
 * default transition duration set by the CSS class.
 * @param {!Element} element The element to restore the transition duration on.
 * @private
 */
warmWelcome.restoreDefaultTransitionDuration_ = function(element) {
  element.style['-webkit-transition-duration'] = '';
};

warmWelcome.restoreDefaultTransitionPropertiesOnAllPages_ = function() {
  for (var i = 0; i < warmWelcome.pageDivs_.length; i++) {
    var div = warmWelcome.pageDivs_[i];
    warmWelcome.restoreDefaultTransitionDuration_(div);
    warmWelcome.enableTransitions_(div, false);
  }
};

/**
 * Enables or disables transitions on webkit transforms for a DOM element.
 * Modelled after javascript/wireless/dom/dom.js:toggleClass.
 * @param {!Element} element The element to set the transition class on.
 * @param {boolean} enabled Whether or not to enable transitions
 * @private
 */
warmWelcome.enableTransitions_ = function(element, enable) {
  var transitionClassRegExp =
      new RegExp('(?:^| +)' + warmWelcome.CSS_TRANSITION_CLASS_ + '(?:$| +)');
  var className = element.className.replace(transitionClassRegExp, ' ');
  element.className =
      className + ' ' + (enable ? warmWelcome.CSS_TRANSITION_CLASS_ : '');
};

/**
 * Removes the CURRENT_PAGE css class from all pages and applies it again to
 * the current page.
 * @private
 */
warmWelcome.resetCurrentPageClass_ = function(pageIndex) {
  var currentPageClassRegExp = new RegExp(
      '(?:^| +)' + warmWelcome.CSS_CURRENT_PAGE_CLASS_ + '(?:$| +)');

  for (var i = 0; i < warmWelcome.pageDivs_.length; i++) {
    var div = warmWelcome.pageDivs_[i];
    div.className = div.className.replace(currentPageClassRegExp, ' ');
    if (i == pageIndex) {
      div.className = div.className + ' ' + warmWelcome.CSS_CURRENT_PAGE_CLASS_;
    }
  }
};

/**
 * Parameter strings for values passed from the app.
 * All of these values need to be synchronized with the values in the app.
 */

/**
 * The name of the parameter identifying if the language is RTL.
 * @private
 * @type {string}
 */
warmWelcome.PARAM_RTL_ = 'RTL';

/**
 * The name of the parameter identifying the welcome sequence to load.
 * @private
 * @type {string}
 */
warmWelcome.PARAM_SEQUENCE_ = 's';

/**
 * Initialize variables associated with recognized parameters from the query
 * portion of the URL.
 * @private
 */
warmWelcome.getSequenceSettingsFromQuery_ = function() {
  // Strip the leading '?' before splitting.
  var params = window.location.search.substring(1).split('&');
  for (var i = 0; i < params.length; i++) {
    var param = params[i].split('=');
    if (param[0] == warmWelcome.PARAM_RTL_) {
      warmWelcome.isRTL_ = param[1] == '1';
    } else if (param[0] == warmWelcome.PARAM_SEQUENCE_) {
      var pageList = param[1].split(',');
      for (var j = 0; j < pageList.length; j++) {
        var pageNumber = parseInt(pageList[j]);
        warmWelcome.pages_[j] = warmWelcome.AVAILABLE_PAGES_[pageNumber];
      }
    }
  }
};

/**
 * The main method loads all translatable content, defines the pages and their
 * relationships to each other, then sends the user to the starting page.
 */
warmWelcome.main = function() {
  warmWelcome.getSequenceSettingsFromQuery_();

  var lastPageIndex = warmWelcome.pages_.length - 1;
  warmWelcome.currDevice_ =
      window.innerWidth < warmWelcome.IPAD_SCREEN_WIDTH_CUTOFF_ ?
          (window.innerHeight < warmWelcome.IPHONE_5_SCREEN_HEIGHT_CUTOFF_ ?
              warmWelcome.DeviceType.IPHONE : warmWelcome.DeviceType.IPHONE_5) :
          warmWelcome.DeviceType.IPAD;

  for (var i = 0; i < warmWelcome.pages_.length; i++) {
    var page = warmWelcome.generatePageText_(warmWelcome.pages_[i],
        warmWelcome.currDevice_);
    if (warmWelcome.isRTL_) {
      // Store pages for RTL in reverse order.
      warmWelcome.pageContent_[lastPageIndex - i] = page;
    } else {
      warmWelcome.pageContent_[i] = page;
    }
  }

  // The following is done in a setTimeout in order for all the preloading being
  // done in the generatePageText_ function to complete before using it. This
  // fixes the problem where the video does not show a preview frame when the
  // buildDocument_ function is called too quickly (b/6871927).
  window.setTimeout(function() {
      warmWelcome.buildDocument_();
      if (warmWelcome.isRTL_) {
        // Go to last page instead of first for RTL languages.
        warmWelcome.setPage(lastPageIndex);
      } else {
        warmWelcome.setPage(0);
      }

      warmWelcome.finishLoading_();
  }, 0);
};

warmWelcome.main();
